@@ -4,10 +4,12 @@ import datetime
 from collections import Counter
 
 from pyg_base import is_bool, try_back, replace, alphabet, ALPHABET, is_str, \
-    as_list, logger, tree_repr, NoneType
-from pyg_mongo._encoders import encode
+    as_list, logger, tree_repr, NoneType, encode
+
 from operator import and_, or_
 from functools import reduce
+from bson import ObjectId
+
 
 __all__ = ['Q','q', 'mdict']
 
@@ -177,7 +179,7 @@ class mkey(object):
         return _q_and([self[key] == value for key, value in kwargs.items()])
     
     def _set(self, other):
-        other = encode(other)
+        other = encode(other, unchanged = ObjectId)
         return mdict({self._key : other})
     
     def __pos__(self):
@@ -203,7 +205,7 @@ class mkey(object):
                 return self._set({_regex : other.pattern, _options: 'i'})
             else:                
                 return self._set({_regex : other.pattern})
-        other = encode(other)
+        other = encode(other, unchanged = ObjectId)
         if isinstance(other, list):
             if len(other) == 1:
                 return self._set({_eq: other[0]})
